//
//  ViewController.m
//  BackgroundFetchTutorial
//
//  Created by MI-MAC01 on 9/23/14.
//  Copyright (c) 2014 Mobisoft. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

@property(nonatomic, strong) UILabel *currentTempLabel;

@property(nonatomic, strong) UILabel *lastUpdatedTimeStamp;

@end

@implementation ViewController

- (void)viewDidLoad {
    
    [super viewDidLoad];
    
    // Initialize View Contents //
    [self initializeViewContents];
    
    // Do any additional setup after loading the view, typically from a nib.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

-(void) initializeViewContents
{
    // Add Temperature Label //
    [self addTempLabel];
    
    // Add TimeStamp Label //
    [self addTimeStampLabel];
    
    // Fetch Current Temp //
    [self fetchTemperature];
}

-(void) addTempLabel
{
    _currentTempLabel = [[UILabel alloc]initWithFrame:CGRectMake(10, 60, 300, 100)];
    
    _currentTempLabel.text = @"Fetching...";
    
    _currentTempLabel.font = [UIFont systemFontOfSize:35];
    
    _currentTempLabel.textColor = TEXTFIELD_COLOR;
    
    _currentTempLabel.backgroundColor = [UIColor clearColor];
    
    _currentTempLabel.textAlignment = NSTextAlignmentCenter;
    
    [self.view addSubview:_currentTempLabel];
    
}

-(void) addTimeStampLabel
{
    _lastUpdatedTimeStamp = [[UILabel alloc]initWithFrame:CGRectMake(10, 200, 300, 80)];
    
    _lastUpdatedTimeStamp.font = [UIFont systemFontOfSize:16];
    
    _lastUpdatedTimeStamp.textColor = TEXTFIELD_COLOR;
    
    _lastUpdatedTimeStamp.backgroundColor = [UIColor clearColor];
    
    _lastUpdatedTimeStamp.textAlignment = NSTextAlignmentCenter;
    
    [self.view addSubview:_lastUpdatedTimeStamp];
    
}

-(NSURLSession *) returnSharedSession
{
    static dispatch_once_t pred = 0;
    
    static NSURLSession *sharedSession = nil;
    
    dispatch_once(&pred, ^{
        
        NSURLSessionConfiguration *sessionConfig = [NSURLSessionConfiguration defaultSessionConfiguration];
        
        sessionConfig.allowsCellularAccess = YES;
        
        sessionConfig.timeoutIntervalForRequest = 30.0;
        
        sessionConfig.timeoutIntervalForResource = 60.0;
        
        sharedSession = [NSURLSession sessionWithConfiguration:sessionConfig];
    });
    
    return sharedSession;
}

-(void) fetchTemperature
{
    if([[UIApplication sharedApplication] applicationState] != UIApplicationStateBackground)
    {
        [DELEGATE beginBackgroundUploadTask];
    }
    else
    {
        DebugLog(@"Background Time:%f",[[UIApplication sharedApplication] backgroundTimeRemaining]);
    }
    
    NSURL *URL = [NSURL URLWithString:@"http://api.openweathermap.org/data/2.5/weather?lat=18.5203&lon=73.8567"];
    
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:URL];
    
    [request setHTTPMethod:@"GET"];
    
    [request setValue:[NSString stringWithFormat:@"application/json"] forHTTPHeaderField:@"Content-Type"];
    
    NSURLSessionDataTask *task = [[self returnSharedSession] dataTaskWithRequest:request
                                                               completionHandler:
                                  ^(NSData *data, NSURLResponse *response, NSError *error) {
                                      
                                      // Parse Response //
                                      [self parseWheatherResponse:response
                                                 withResponseData:data
                                                         andError:error];
                                      
                                  }];
    
    [task resume];
    
    if(DELEGATE.updateTimer == nil)
    {
        [self initializeUpdateTimer];
    }
}

-(void) parseWheatherResponse:(NSURLResponse *)response
             withResponseData:(NSData *) reponseData
                     andError:(NSError *) error
{
    NSMutableDictionary *responseDict =  [NSJSONSerialization
                                          JSONObjectWithData:reponseData
                                          options:kNilOptions
                                          error:&error];
    
    DebugLog(@"%@",responseDict);
    
    NSHTTPURLResponse* httpResponse = (NSHTTPURLResponse*)response;
    
    if([httpResponse statusCode] == 200)
    {
        NSDictionary *tempDict = [responseDict objectForKey:@"main"];
        
        [self setCurrentTemp:[tempDict objectForKey:@"temp"]];
        
        DebugLog(@"%@",tempDict);
        
    }else {
        
        dispatch_async(dispatch_get_main_queue(), ^{
            
            // Update the UI on the main thread //
            
            _currentTempLabel.text = @"Fetching Failuree...";
            
            [self updatelastUpdatedTimeStamp];
            
        });


    }
}

-(void) setCurrentTemp:(NSString *) temp
{
    // Current Temperature is in Kelvin Convert it to Celsius and then Update the Current Temperatue Label //
    
    NSNumberFormatter *nsNumberFormatter = [[NSNumberFormatter alloc] init];
    [nsNumberFormatter setMaximumFractionDigits:1];
    [nsNumberFormatter stringFromNumber:[self tempToCelcius:[NSNumber numberWithDouble:[temp doubleValue]]]];
    
    DebugLog(@"%@",[nsNumberFormatter stringFromNumber:[self tempToCelcius:[NSNumber numberWithDouble:[temp doubleValue]]]]);
    
    // Update UI after computation //
    
    dispatch_async(dispatch_get_main_queue(), ^{
        
        // Update the UI on the main thread //
        
        _currentTempLabel.text = [NSString stringWithFormat:@"%@ °C",[nsNumberFormatter stringFromNumber:[self tempToCelcius:[NSNumber numberWithDouble:[temp doubleValue]]]]];
        
        [self updatelastUpdatedTimeStamp];
        
    });
    
}

-(void) updatelastUpdatedTimeStamp
{
    // Initialize Dateformatter //
    NSDateFormatter* dateformatter = [[NSDateFormatter alloc]init];
    
    [dateformatter setDateFormat:@"MMM dd HH:mm:ss"];
    
    _lastUpdatedTimeStamp.text = [NSString stringWithFormat:@"Last Updated: %@",[dateformatter stringFromDate:[NSDate date]]] ;
}

-(NSNumber *) tempToCelcius:(NSNumber *) tempKelvin
{
    return @(tempKelvin.floatValue - 273.15);
}

-(void) initializeUpdateTimer
{
    DELEGATE.updateTimer = [NSTimer scheduledTimerWithTimeInterval:10
                                                            target:self
                                                          selector:@selector(fetchTemperature)
                                                          userInfo:nil repeats:YES];
}

@end
